@@ -13,9 +13,72 @@ typedef int Py_ssize_t;
 #define PY_SSIZE_T_MIN INT_MIN
 #endif
 
+/** @todo Remove Python 3 defines and update the code when Python 2 is out of the picture. */
+
+// Python 3 has unified int & long types.
+#if PY_MAJOR_VERSION >= 3
+#define PyInt_AS_LONG PyLong_AS_LONG
+#define PyInt_Check PyLong_Check
+#define PyInt_FromLong PyLong_FromLong
+#endif
+
+// Python 3 has removed PyString and related functions, in favor of PyBytes & PyUnicode.
+#if PY_MAJOR_VERSION > 3 || (PY_MAJOR_VERSION == 3 && PY_MINOR_VERSION >= 3)
+#define PyString_Check PyUnicode_Check
+#define PyString_FromFormat PyUnicode_FromFormat
+#define PyString_FromString PyUnicode_FromString
+#define PyString_AsString PyUnicode_AsUTF8
+static Py_ssize_t PyString_Size(PyObject *string) {
+	Py_ssize_t size;
+	char *ret = PyUnicode_AsUTF8AndSize(string, &size);
+	if (! ret) {
+		return -1;
+	}
+	return size;
+}
+#define get_pystring PyUnicode_AsUTF8AndSize
+#define PyStringFree(string) ;
+#elif PY_MAJOR_VERSION >= 3
+#define PyString_Check PyUnicode_Check
+#define PyString_FromFormat PyUnicode_FromFormat
+#define PyString_FromString PyUnicode_FromString
+static char *PyString_AsString(PyObject *string) {
+	PyObject *bytes = PyUnicode_AsUTF8String(string);
+	char *ret = NULL;
+	if (! bytes)
+		return NULL;
+	ret = g_strdup(PyBytes_AsString(bytes));
+	Py_DECREF(bytes);
+	return ret;
+}
+static char *get_pystring(PyObject *string, Py_ssize_t *size) {
+	char *ret = NULL;
+
+	ret = PyString_AsString(string);
+	*size = strlen(ret);
+	return ret;
+}
+#define PyStringFree(string) g_free(string)
+#elif PY_MAJOR_VERSION >= 3
+#else
+static char *get_pystring(PyObject *string, Py_ssize_t *size) {
+	char *ret = NULL;
+
+	PyString_AsStringAndSize(string, &ret, size);
+
+	return ret;
+}
+#define PyStringFree(string) ;
+#endif
+
 GQuark lasso_wrapper_key;
 
+
+#if PY_MAJOR_VERSION > 3
+PyMODINIT_FUNC PyInit_lasso(void);
+#else
 PyMODINIT_FUNC init_lasso(void);
+#endif
 G_GNUC_UNUSED static PyObject* get_pystring_from_xml_node(xmlNode *xmlnode);
 G_GNUC_UNUSED static xmlNode*  get_xml_node_from_pystring(PyObject *string);
 G_GNUC_UNUSED static PyObject* get_dict_from_hashtable_of_objects(GHashTable *value);
@@ -85,46 +148,60 @@ get_dict_from_hashtable_of_strings(GHashTable *value)
 
 	dict = PyDict_New();
 
-	begin = keys = g_hash_table_get_keys(value);
-	for (; keys; keys = g_list_next(keys)) {
-		item_value = g_hash_table_lookup(value, keys->data);
-		if (item_value) {
-			item = PyString_FromString(item_value);
-			PyDict_SetItemString(dict, (char*)keys->data, item);
-			Py_DECREF(item);
-		} else {
-			PyErr_Warn(PyExc_RuntimeWarning, "hashtable contains a null value");
+	if (value) {
+		begin = keys = g_hash_table_get_keys(value);
+		for (; keys; keys = g_list_next(keys)) {
+			item_value = g_hash_table_lookup(value, keys->data);
+			if (item_value) {
+				item = PyString_FromString(item_value);
+				PyDict_SetItemString(dict, (char*)keys->data, item);
+				Py_DECREF(item);
+			} else {
+				PyErr_Warn(PyExc_RuntimeWarning, "hashtable contains a null value");
+			}
 		}
+		g_list_free(begin);
 	}
-	g_list_free(begin);
 
 	proxy = PyDictProxy_New(dict);
 	Py_DECREF(dict);
 	return proxy;
 }
 
+static xmlBuffer*
+xmlnode_to_xmlbuffer(xmlNode *node)
+{
+	xmlOutputBufferPtr output_buffer;
+	xmlBuffer *buffer;
+
+	if (! node)
+		return NULL;
+
+	buffer = xmlBufferCreate();
+	output_buffer = xmlOutputBufferCreateBuffer(buffer, NULL);
+	xmlNodeDumpOutput(output_buffer, NULL, node, 0, 0, NULL);
+	xmlOutputBufferClose(output_buffer);
+	xmlBufferAdd(buffer, BAD_CAST "", 1);
+
+	return buffer;
+}
+
 static PyObject*
 get_pystring_from_xml_node(xmlNode *xmlnode)
 {
-	xmlOutputBufferPtr buf;
 	PyObject *pystring = NULL;
+	xmlBuffer *buffer;
 
 	if (xmlnode == NULL) {
 		return NULL;
 	}
+	buffer = xmlnode_to_xmlbuffer(xmlnode);
 
-	buf = xmlAllocOutputBuffer(NULL);
-	if (buf == NULL) {
+	if (buffer == NULL) {
 		pystring = NULL;
 	} else {
-		xmlNodeDumpOutput(buf, NULL, xmlnode, 0, 1, NULL);
-		xmlOutputBufferFlush(buf);
-		if (buf->conv == NULL) {
-			pystring = PyString_FromString((char*)buf->buffer->content);
-		} else {
-			pystring = PyString_FromString((char*)buf->conv->content);
-		}
-		xmlOutputBufferClose(buf);
+		pystring = PyString_FromString((char*)xmlBufferContent(buffer));
+		xmlBufferFree(buffer);
 	}
 
 	return pystring;
@@ -183,7 +260,7 @@ set_hashtable_of_pygobject(GHashTable *a_hash, PyObject *dict) {
 	g_hash_table_remove_all (a_hash);
 	i = 0;
 	while (PyDict_Next(dict, &i, &key, &value)) {
-		char *ckey = g_strdup(PyString_AsString(key));
+		char *ckey = PyString_AsString(key);
 		g_hash_table_replace (a_hash, ckey, ((PyGObjectPtr*)value)->obj);
 	}
 	return;
@@ -324,8 +401,13 @@ failure:
 
 static xmlNode*
 get_xml_node_from_pystring(PyObject *string) {
-	return lasso_string_fragment_to_xmlnode(PyString_AsString(string),
-			PyString_Size(string));
+	char *utf8 = NULL;
+	Py_ssize_t size;
+	xmlNode *xml_node;
+	utf8 = get_pystring(string, &size);
+	xml_node = lasso_string_fragment_to_xmlnode(utf8, size);
+	PyStringFree(utf8);
+	return xml_node;
 }
 
 /** Return a tuple containing the string contained in a_list */
@@ -473,7 +555,7 @@ PyGObjectPtr_dealloc(PyGObjectPtr *self)
 	g_object_set_qdata_full(self->obj, lasso_wrapper_key, NULL, NULL);
 	g_object_unref(self->obj);
 	Py_XDECREF(self->typename);
-	self->ob_type->tp_free((PyObject*)self);
+	Py_TYPE(self)->tp_free((PyObject*)self);
 }
 
 static int
@@ -545,8 +627,7 @@ static PyGetSetDef PyGObjectPtr_getseters[] = {
 
 
 static PyTypeObject PyGObjectPtrType = {
-	PyObject_HEAD_INIT(NULL)
-	0, /* ob_size */
+	PyVarObject_HEAD_INIT(NULL, 0 /* ob_size */)
 	"_lasso.PyGObjectPtr", /* tp_name */
 	sizeof(PyGObjectPtr),  /* tp_basicsize */
 	0,                     /* tp_itemsize */
@@ -574,7 +655,7 @@ static PyTypeObject PyGObjectPtrType = {
 	0,       /* tp_iter */
 	0,       /* tp_iternext */
 	0,       /* tp_methods */
-	PyGObjectPtr_members,   /* tp_members */
+	.tp_members=PyGObjectPtr_members,   /* tp_members */
 	PyGObjectPtr_getseters, /* tp_getset */
 	NULL,
 	NULL
@@ -593,7 +674,7 @@ set_object_field(GObject **a_gobject_ptr, PyGObjectPtr *a_pygobject) {
 }
 
 
-static PyObject *get_logger_object() {
+static PyObject *get_logger_object(const char *domain) {
 	static PyObject *_logger_object = NULL;
 
 	PyObject *logging_module = PyImport_ImportModule("lasso");
@@ -606,12 +687,13 @@ static PyObject *get_logger_object() {
 	}
 	/* XXX: needed so that PyImport_ImportModule("logging") always works */
 	logging_module = PyImport_ImportModule("sys");
-	if (logging_module)
+	if (logging_module) {
 		Py_DECREF(logging_module);
+	}
 	logging_module = PyImport_ImportModule("logging");
 	if (logging_module) {
 		_logger_object = PyObject_CallMethod(logging_module, "getLogger",
-				"s#", "lasso", sizeof("lasso")-1);
+				"s#", domain, strlen(domain));
 		Py_DECREF(logging_module);
 	}
 exit:
@@ -623,10 +705,10 @@ exit:
 }
 
 static void
-lasso_python_log(G_GNUC_UNUSED const char *domain, GLogLevelFlags log_level, const gchar *message,
+lasso_python_log(const char *domain, GLogLevelFlags log_level, const gchar *message,
 		G_GNUC_UNUSED gpointer user_data)
 {
-	PyObject *logger_object = get_logger_object(), *result;
+	PyObject *logger_object = get_logger_object(domain), *result;
 	char *method = NULL;
 
 	if (! logger_object) {
