@@ -18,8 +18,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 /**
@@ -82,11 +81,11 @@
 #include "../id-ff/provider.h"
 #include "../id-ff/providerprivate.h"
 
-#include "./discovery.h"
+#include "discovery.h"
 #include "../xml/id-wsf-2.0/idwsf2_strings.h"
-#include "./soap_binding.h"
-#include "./idwsf2_helper.h"
-#include "./saml2_login.h"
+#include "soap_binding.h"
+#include "idwsf2_helper.h"
+#include "saml2_login.h"
 #include "../utils.h"
 
 struct _LassoIdWsf2DiscoveryPrivate
