@@ -18,12 +18,11 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "../utils.h"
-#include "./authentication.h"
+#include "authentication.h"
 #include "../xml/idwsf_strings.h"
 #include "../xml/private.h"
 #include "../xml/sa_sasl_request.h"
