@@ -53,11 +53,11 @@
 static struct XmlSnippet schema_snippets[] = {
 	{ "Condition", SNIPPET_LIST_NODES,
 		G_STRUCT_OFFSET(LassoSaml2Conditions, Condition), NULL, NULL, NULL},
-	{ "AudienceRestriction", SNIPPET_LIST_NODES,
+	{ "AudienceRestriction", SNIPPET_LIST_NODES | SNIPPET_JUMP_ON_MATCH | SNIPPET_BACK_1,
 		G_STRUCT_OFFSET(LassoSaml2Conditions, AudienceRestriction), NULL, NULL, NULL},
-	{ "OneTimeUse", SNIPPET_LIST_NODES,
+	{ "OneTimeUse", SNIPPET_LIST_NODES | SNIPPET_JUMP_ON_MATCH | SNIPPET_BACK_2,
 		G_STRUCT_OFFSET(LassoSaml2Conditions, OneTimeUse), NULL, NULL, NULL},
-	{ "ProxyRestriction", SNIPPET_LIST_NODES,
+	{ "ProxyRestriction", SNIPPET_LIST_NODES | SNIPPET_JUMP_ON_MATCH | SNIPPET_BACK_3,
 		G_STRUCT_OFFSET(LassoSaml2Conditions, ProxyRestriction), NULL, NULL, NULL},
 	{ "NotBefore", SNIPPET_ATTRIBUTE,
 		G_STRUCT_OFFSET(LassoSaml2Conditions, NotBefore), NULL, NULL, NULL},
