@@ -18,8 +18,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "../private.h"
@@ -48,7 +47,7 @@
 
 
 static struct XmlSnippet schema_snippets[] = {
-	{ "IDPEntry", SNIPPET_NODE,
+	{ "IDPEntry", SNIPPET_LIST_NODES,
 		G_STRUCT_OFFSET(LassoSamlp2IDPList, IDPEntry), NULL, NULL, NULL},
 	{ "GetComplete", SNIPPET_CONTENT,
 		G_STRUCT_OFFSET(LassoSamlp2IDPList, GetComplete), NULL, NULL, NULL},
