@@ -57,8 +57,9 @@ extern LassoNode* lasso_assertion_encrypt(LassoSaml2Assertion *assertion, char *
 static struct XmlSnippet schema_snippets[] = {
 	{ "Assertion", SNIPPET_LIST_NODES, G_STRUCT_OFFSET(LassoSamlp2Response, Assertion), NULL,
 		LASSO_SAML2_ASSERTION_PREFIX, LASSO_SAML2_ASSERTION_HREF},
-	{ "EncryptedAssertion", SNIPPET_LIST_NODES, G_STRUCT_OFFSET(LassoSamlp2Response,
-			EncryptedAssertion), NULL, LASSO_SAML2_ASSERTION_PREFIX,
+	{ "EncryptedAssertion", SNIPPET_LIST_NODES | SNIPPET_JUMP_ON_MATCH | SNIPPET_BACK_1,
+		G_STRUCT_OFFSET(LassoSamlp2Response, EncryptedAssertion),
+		NULL, LASSO_SAML2_ASSERTION_PREFIX,
 	LASSO_SAML2_ASSERTION_HREF},
 	{NULL, 0, 0, NULL, NULL, NULL}
 };
